import * as grpc from 'grpc';
import { ClientGRPC } from '../grpc/grpcClient';
import { getKey, logger, UserInfo, generateJWT, commonAppConst } from 'skyeye-svc-common-utils';
import { GrpcClientInstance } from '../interfaces/grpcClientInstance';
import { GrpcServiceResponse } from '../interfaces/grpcServiceResponse';
import { grpcConst } from '../utils/grpcConst';
import { GrpcListRequest } from '../interfaces/grpcListRequest';
const ProtoPath = __dirname + grpcConst.protoPath + grpcConst.proto.emailtemplate;

export class EmailTemplateClient {
    private static instance: EmailTemplateClient;
    private static emailTemplateServer: any;
    private static metaData: grpc.Metadata;
    private static options: grpc.CallOptions;
    private constructor() { }
    public static async getInstance(userInfo: UserInfo): Promise<EmailTemplateClient> {
        if (!EmailTemplateClient.instance) {
            EmailTemplateClient.instance = new EmailTemplateClient();
            let clientGrpc: ClientGRPC = new ClientGRPC();
            let clientInit: GrpcClientInstance = await clientGrpc.initClient(ProtoPath);

            let EmailTemplateService: any = grpc.loadPackageDefinition(clientInit.loadPackageDefinition);
            EmailTemplateService = EmailTemplateService.hs[grpcConst.service.emailtemplate].Service;

            let clientConnection = await getKey(grpcConst.hostName.emailtemplate) || grpcConst.defaultGRPCAddress;
            this.emailTemplateServer = new EmailTemplateService(`${clientConnection}`, grpc.credentials.createInsecure());
            logger.info(`started ${grpcConst.hostName.emailtemplate} connection : ${clientConnection}`);
        }

        EmailTemplateClient.metaData = new grpc.Metadata();
        if (userInfo) {
            let token: string = `${commonAppConst.httpHeader.BEARER} ${await generateJWT(userInfo)}`;
            EmailTemplateClient.metaData.add(commonAppConst.httpHeader.AUTHORIZATION, token);
        }
        let deadline = Number(await getKey(grpcConst.deadline.short));
        deadline = new Date().setSeconds(new Date().getSeconds() + deadline);
        EmailTemplateClient.options = { deadline: deadline };
        return EmailTemplateClient.instance;
    }

    async root(): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.root`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.root(
                {},
                EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error)
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.root`)
                })
        });
    }

    async emailTemplateCreate(code: string, name: string, description: string, subject: string, body: string): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateCreate`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.emailTemplateCreate(
                { code: code, name: name, description: description, subject: subject, body: body }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateCreate`)
                })
        })
    }

    async emailTemplateDelete(guid: string, deletedStatus?: number): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateDelete`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.emailTemplateDelete(
                { guid: guid, deletedStatus: deletedStatus }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateDelete`)
                })
        })
    }

    async emailTemplateRetrieve(guid: string): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateRetrieve`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.emailTemplateRetrieve(
                { guid: guid }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateRetrieve`)
                })
        })
    }

    async emailTemplateList(code: string = null, name: string = null, subject: string = null, deletedStatus: number = null, listProps: GrpcListRequest): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateList`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.EmailTemplateList(
                { code: code, name: name, subject: subject, deletedStatus: deletedStatus, ...listProps }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateList`)
                })
        })
    }

    async emailTemplateRetrieveByIds(guids: string[], deletedStatus: number[]): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateRetrieveByIds`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.emailTemplateRetrieveByIds(
                { guids: guids, deletedStatus: deletedStatus }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateRetrieveByIds`)
                })
        })
    }

    async emailTemplateUpdate(guid: string, code: string, name: string, description: string, subject: string, body: string): Promise<GrpcServiceResponse> {
        logger.info(`start EmailTemplateClient.emailTemplateUpdate`)
        return new Promise((resolve) => {
            EmailTemplateClient.emailTemplateServer.emailTemplateUpdate(
                { guid: guid, code: code, name: name, description: description, subject: subject, body: body }, EmailTemplateClient.metaData,
                EmailTemplateClient.options,
                (error: any, response: GrpcServiceResponse) => {
                    if (error) {
                        logger.error(error);
                    }
                    resolve(response);
                    logger.info(`end EmailTemplateClient.emailTemplateUpdate`)
                })
        })
    }
}


