import * as grpc from 'grpc';
import * as protoLoader from '@grpc/proto-loader';
import { logger, getKey } from 'skyeye-svc-common-utils';
import { grpcConfig } from '../utils/grpcConfig';
import { GrpcServerInstance } from '../interfaces/grpcServerInstance';
import { grpcConst } from '../utils/grpcConst';

export class ServerGRPC {
    private server: grpc.Server;
    async bindServer(protoPath: string | Array<string>): Promise<GrpcServerInstance> {

        logger.info('start src/grc/grpcService.bindServer');
        if (!protoPath || protoPath.length == 0) {
            let error = new Error('FAILED_PRECONDITION: Grpc Server protoPath not found.');
            logger.error(`ErrorHandler: Server Grpc error: ${error.message}`);
            throw error;
        }

        const packageDef: protoLoader.PackageDefinition = protoLoader.loadSync(
            protoPath,
            {
                keepCase: true,
                longs: String,
                enums: String,
                defaults: true,
                oneofs: true,
                arrays: true,
                objects: true
            });
        const protoDescriptor = grpc.loadPackageDefinition(packageDef);
        this.server = new grpc.Server();
        let serverConnection = await getKey(grpcConfig.GrpcServerHost) || grpcConst.defaultGRPCAddress;
        this.server.bind(`${serverConnection}`, grpc.ServerCredentials.createInsecure());
        logger.info(`${grpcConfig.GrpcServerHost} grpc server running at ${serverConnection}`);
        logger.info('end src/grc/grpcService.bindServer');
        return { protoDescriptor: protoDescriptor };
    }

    async startServer(service: any, listOfFunction: any) {
        this.server.addService(service, listOfFunction);
        this.server.start();
    }
}
async function buildProtoFilePath(serviceName: string): Promise<string | Array<string>> {
    logger.info('start src/grc/grpcService.buildProtoFilePath');
    let result: any;
    let basePath = __dirname + grpcConst.protoPath;
    switch (serviceName) {
        case grpcConst.service.rbac: {
            result = basePath + grpcConst.proto.rbac;
            break;
        }
        case grpcConst.service.subscription: {
            result = basePath + grpcConst.proto.subscription;
            break;
        }
        case grpcConst.service.user: {
            result = basePath + grpcConst.proto.user;
            break;
        }
        case grpcConst.service.organisation: {
            result = basePath + grpcConst.proto.organisation;
            break;
        }
        case grpcConst.service.campaign: {
            result = basePath + grpcConst.proto.campaign;
            break;
        }
        case grpcConst.service.scoutai: {
            result = basePath + grpcConst.proto.scoutai;
            break;
        }
        case grpcConst.service.emailtemplate: {
            result = basePath + grpcConst.proto.emailtemplate;
            break;
        }
        case grpcConst.service.search:{
            result = basePath + grpcConst.proto.search
            break;
        }
        default: {
            result = undefined;
            break;
        }
    }
    logger.info(`end src/grc/grpcService.buildProtoFilePath: ${result}`);
    return result;
}

export async function startGrpcServer(serviceName: string, listOfFunction: any) {
    logger.info(`start src/grc/grpcService.startGrpcServer : ${serviceName}`);
    let server = new ServerGRPC();
    let instance: GrpcServerInstance = await server.bindServer(await buildProtoFilePath(serviceName));

    let service: any = instance.protoDescriptor;
    service = service.hs[serviceName].Service.service;
    await server.startServer(service, listOfFunction);
    logger.info('start src/grc/grpcService.startGrpcServer');
}
